package com.cnbm.packing.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.packing.dto.IdVo;
import com.cnbm.packing.dto.ProductLevelDTO;
import com.cnbm.packing.entity.ProductLevel;
import com.cnbm.packing.excel.ProductLevelExcel;
import com.cnbm.packing.mapper.ProductLevelMapper;
import com.cnbm.packing.service.ProductLevelServiceBiz;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 产品等级 表  前端控制器
 *
 * @author codeGenerator
 * @since  2023-12-21
 */
@RestController
@RequestMapping("/basic/productLevel")
@Api(tags="产品等级 表")
public class ProductLevelController {
    @Autowired
    private ProductLevelServiceBiz productLevelService;
    @Autowired
    private ProductLevelMapper productLevelMapper;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "name", value = "产品等级名", paramType = "query", dataTypeClass = String.class)
    })
//    @PreAuthorize("@ex.hasAuthority('basic:productLevel:page')")
    public Result<PageData<ProductLevelDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<ProductLevelDTO> page = productLevelService.page(params);

        return new Result<PageData<ProductLevelDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
//    @PreAuthorize("@ex.hasAuthority('basic:productLevel:info')")
    public Result<ProductLevelDTO> get(@PathVariable("id") Long id){
        ProductLevelDTO data = productLevelService.get(id);

        return new Result<ProductLevelDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
//    @PreAuthorize("@ex.hasAuthority('basic:productLevel:save')")
    public Result<IdVo> save(@RequestBody ProductLevelDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        //验证编码是否重名
        QueryWrapper<ProductLevel> wrapper1 = new QueryWrapper<>();
        wrapper1.eq(ProductLevel.CODE, dto.getCode());
        if(productLevelMapper.selectCount(wrapper1)!= 0){
            return new Result().error(1,"编码重复");
        }
        //验证名称是否重名
        QueryWrapper<ProductLevel> wrapper2 = new QueryWrapper<>();
        wrapper2.eq(ProductLevel.PRODUCT_LEVEL, dto.getProductLevel());
        if(productLevelMapper.selectCount(wrapper2)!= 0){
            return new Result().error(1,"名称重复");
        }
        else {
            return new Result<IdVo>().ok(productLevelService.add(dto));
        }
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
//    @PreAuthorize("@ex.hasAuthority('basic:productLevel:update')")
    public Result<IdVo> update(@RequestBody ProductLevelDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        //验证编码是否重名
        ProductLevelDTO entity = productLevelService.get(dto.getId());
        QueryWrapper<ProductLevel> wrapper1 = new QueryWrapper<>();
        wrapper1.eq(ProductLevel.CODE, dto.getCode());
        if(entity.getCode()!=dto.getCode() && productLevelMapper.selectCount(wrapper1)!= 0){
            return new Result().error(1,"编码重复");
        }
        //验证名称是否重名
        QueryWrapper<ProductLevel> wrapper2 = new QueryWrapper<>();
        wrapper2.eq(ProductLevel.PRODUCT_LEVEL, dto.getProductLevel());
        if(entity.getProductLevel()!=dto.getProductLevel() && productLevelMapper.selectCount(wrapper2)!= 0){
            return new Result().error(1,"名称重复");
        }
        else {
            return new Result<IdVo>().ok(productLevelService.edit(dto));
        }
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
//    @PreAuthorize("@ex.hasAuthority('basic:productLevel:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        productLevelService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
//    @PreAuthorize("@ex.hasAuthority('basic:productLevel:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<ProductLevelDTO> list = productLevelService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, ProductLevelExcel.class);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取产品等级列表")
    public List<ProductLevelDTO> list(){
        return productLevelService.list();
    }
}