package com.cnbm.packing.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.packing.dto.IdVo;
import com.cnbm.packing.dto.WorkingTimeDTO;
import com.cnbm.packing.entity.WorkingTime;
import com.cnbm.packing.excel.WorkingTimeExcel;
import com.cnbm.packing.mapper.WorkingTimeMapper;
import com.cnbm.packing.service.WorkingTimeServiceBiz;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;


/**
 * 班次时间段 表  前端控制器
 *
 * @author packingGenerator
 * @since  2023-12-19
 */
@RestController
@RequestMapping("/packing/workingTime")
@Api(tags="班次时间段 表")
public class WorkingTimeController {
    @Autowired
    private WorkingTimeServiceBiz workingTimeService;
    @Autowired
    private WorkingTimeMapper workingTimeMapper;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = "name", value = "班次名", paramType = "query", dataTypeClass = String.class)
    })
    public Result<PageData<WorkingTimeDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<WorkingTimeDTO> page = workingTimeService.page(params);

        return new Result<PageData<WorkingTimeDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    public Result<WorkingTimeDTO> get(@PathVariable("id") Long id){
        WorkingTimeDTO data = workingTimeService.get(id);

        return new Result<WorkingTimeDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    public Result<IdVo> save(@RequestBody WorkingTimeDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        //验证编码是否重名
        QueryWrapper<WorkingTime> wrapper1 = new QueryWrapper<>();
        wrapper1.eq(WorkingTime.CODE, dto.getCode());
        if(workingTimeMapper.selectCount(wrapper1)!= 0){
            return new Result().error(1,"编码重复");
        }
        //验证名称是否重名
        QueryWrapper<WorkingTime> wrapper2 = new QueryWrapper<>();
        wrapper2.eq(WorkingTime.ORDER_NAME, dto.getOrderName());
        if(workingTimeMapper.selectCount(wrapper2)!= 0){
            return new Result().error(1,"名称重复");
        }
        else {
            return new Result<IdVo>().ok(workingTimeService.add(dto));
        }
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    public Result<IdVo> update(@RequestBody WorkingTimeDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        //验证编码是否重名
        WorkingTimeDTO entity = workingTimeService.get(dto.getId());
        QueryWrapper<WorkingTime> wrapper1 = new QueryWrapper<>();
        wrapper1.eq(WorkingTime.CODE, dto.getCode());
        if(entity.getCode()!=dto.getCode() && workingTimeMapper.selectCount(wrapper1)!= 0){
            return new Result().error(1,"编码重复");
        }
        //验证名称是否重名
        QueryWrapper<WorkingTime> wrapper2 = new QueryWrapper<>();
        wrapper2.eq( WorkingTime.ORDER_NAME, dto.getOrderName());
        if(entity.getOrderName()!=dto.getOrderName() && workingTimeMapper.selectCount(wrapper2)!= 0){
            return new Result().error(1,"名称重复");
        }
        else {
            return new Result<IdVo>().ok(workingTimeService.edit(dto));
        }
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        workingTimeService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<WorkingTimeDTO> list = workingTimeService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, WorkingTimeExcel.class);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取班次列表")
    public List<WorkingTimeDTO> list(){
        return workingTimeService.list();
    }

    @GetMapping("getOrderName")
    @ApiOperation("查询时间匹配的班次名称")
//    @ApiImplicitParam(name = "time", value = "时间", paramType = "query", required = true, dataTypeClass = LocalDateTime.class)
    public Result<String> getOrderName(@RequestParam @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") LocalDateTime time){
        String orderName = workingTimeService.getOrderName(time);

        return new Result<String>().ok(orderName);
    }

}